<?php

namespace Drupal\profile_manager\EventSubscriber;

use Drupal\Core\Config\ConfigEvents;
use Drupal\Core\Config\StorageTransformEvent;
use Drupal\Core\Serialization\Yaml;
use Drupal\profile_manager\OptionalModulesTrait;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Ensures optional module configurations are imported during a config import.
 */
class OptionalModuleConfigImport implements EventSubscriberInterface {

  use OptionalModulesTrait;

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[ConfigEvents::STORAGE_TRANSFORM_IMPORT][] = ['onImportTransform', 100];
    return $events;
  }

  /**
   * Ensure that the optional module config is imported.
   */
  public function onImportTransform(StorageTransformEvent $event) {
    $storage = $event->getStorage();
    foreach ($this->getProfileOptionalModulePaths() as $module_path) {
      $config = $module_path . DIRECTORY_SEPARATOR . "config";
      $directories = [
        $config . DIRECTORY_SEPARATOR . "install",
        $config . DIRECTORY_SEPARATOR . "optional",
      ];
      foreach ($directories as $dir) {
        if (!file_exists($dir)) {
          continue;
        }
        foreach (scandir($dir) as $file_name) {
          $info = pathinfo($file_name);
          if ($info['extension'] === 'yml') {
            $config = Yaml::decode(file_get_contents($dir . DIRECTORY_SEPARATOR . $file_name));
            $storage->write($info['filename'], $config);
          }
        }
      }
    }
  }

}
