<?php

namespace Drupal\profile_manager;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 * Provides simplified access to optional module details like name and path.
 */
trait OptionalModulesTrait {

  /**
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $profileManagerConfig;

  /**
   * The module handler object.
   *
   * @var ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $handler;

  /**
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $handler
   */
  public function __construct(ConfigFactoryInterface $config_factory, ModuleHandlerInterface $handler) {
    $this->profileManagerConfig = $config_factory->get('profile_manager.settings');
    $this->handler = $handler;
  }

  /**
   * Get a list of enabled "allowed_modules" directory paths.
   *
   * @return array
   */
  protected function getProfileOptionalModulePaths(): array {
    $paths = [];
    $allowed_modules = $this->profileManagerConfig->get('allowed_modules') ?? [];
    foreach (array_keys($allowed_modules) as $module_name) {
      if (!$this->handler->moduleExists($module_name)) {
        continue;
      }
      $extension = $this->handler->getModule($module_name);
      $paths[] = realpath($extension->getPath());
    }
    return $paths;
  }

  /**
   * Get a list of enabled "allowed_modules" present in the codebase.
   *
   * @return array
   */
  protected function getProfileOptionalModules(): array {
    $modules = [];
    $allowed_modules = $this->profileManagerConfig->get('allowed_modules') ?? [];
    foreach (array_keys($allowed_modules) as $module_name) {
      if (!$this->handler->moduleExists($module_name)) {
        continue;
      }
      // @todo handle module weight?
      // Since this is going to be used to add/remove from core.extension
      // during the import/export process, we should find and preserve module
      // weight.
      $modules[] = $module_name;
    }
    return $modules;
  }

}
